import io
import html
import os.path
import codecs

import uno
import unohelper

from com.sun.star.awt.FontWeight import (NORMAL, BOLD)
from com.sun.star.awt.FontUnderline import (SINGLE, NONE)
from com.sun.star.awt.FontSlant import (NONE, ITALIC)

from com.sun.star.task import XJobExecutor

class RusLinuxJob(unohelper.Base, XJobExecutor):
	def __init__(self, ctx):
		'''Конструктор класса'''
		# Сохранение контекста компонента для последующего использования 
		self.ctx = ctx
	
	def replaceSpecialSymbol(self, error):
		'''Функция замены специальных символов'''
		# Специальные символы и их эквиваленты
		specialsymbols = {'—' : '-', '«' : '"', '»' : '"'}
		# Результирующий фрагмент строки
		outfragment = ''
		# Обход проблемных символов
		for symnum in range(error.end - error.start):
			symbol = str(error.object)[error.start + symnum]
			if (symbol in specialsymbols):
				# Замена символа в том случае, если он известен
				outfragment = outfragment + specialsymbols[symbol]
			else:
				# Замена неизвестного символа на символ пробела
				outfragment = outfragment + ' '
		# Возврат результирующего фрагмента строки и позиции для продолжения перекодировки
		return (outfragment, error.end)
	
	def convertDate(self, date):
		'''Функция преобразования даты в строковый формат'''
		# Список названий месяцев
		months = ['января', 'февраля', 'марта', 'апреля', 'мая', 'июня', 'июля', 'августа', 'сентября', 'октября', 'ноября', 'декабря']
		if date is not None:
			try:
				# Преобразование переданной даты
				resdate = str(date.Day) + ' ' + months[date.Month - 1] + ' ' + str(date.Year)  + ' г.'
			except:
				# Преобразование текущей даты 
				date = datetime.datetime.now()
				resdate = str(date.day) + ' ' + months[date.month - 1] + ' ' + str(date.year)  + ' г.'
		else:
			# Преобразование текущей даты
			date = datetime.datetime.now()
			resdate = str(date.day) + ' ' + months[date.month - 1] + ' ' + str(date.year)  + ' г.'
		
		return resdate
	
	def stringEqualsWithoutCase(self, string1, string2):
		'''Функция сравнения строк без учета регистра'''
		try:
			# Сравнение строк в нижнем регистре
			return string1.lower() == string2.lower()
		except AttributeError:
			# Простое сравнение в случае неудачи
			return string1 == string2
	
	def trigger(self, event):
		'''Обработчик события'''
		# Получение объекта рабочего стола
		desktop = self.ctx.ServiceManager.createInstanceWithContext("com.sun.star.frame.Desktop", self.ctx)
		# Получение объекта текущего документа
		document = desktop.getCurrentComponent()
		# Проверка возможности доступа к тексту документа
		if not hasattr(document, "Text"):
			return
		# Получение пути к результирующему файлу
		srcurl = document.URL
		try:
			srcpath = uno.fileUrlToSystemPath(srcurl)
			htmlfilename = os.path.splitext(srcpath)[0] + '.html'
		except:
			htmlfilename = 'output.html'
		# Регистрация обработчика для замены специальных символов
		codecs.register_error('replacespecialsymbol', self.replaceSpecialSymbol)
		# Открытие результирующего файла в режиме записи
		htmlfile = io.open(htmlfilename, 'w', encoding='koi8-r',errors='replacespecialsymbol')
		# Открытие документа HTML
		# Форматирование списка ключевых слов
		separator = ''
		keywords = ''
		for keyword in document.DocumentProperties.Keywords:
			keywords = keywords + separator + keyword
			separator = ', '
		# Форматирование переданных пользователем данных
		origauthor = 'Unknown'
		origname = 'Unknown'
		origurl = ''
		origdate = ''
		userprops = document.DocumentProperties.getUserDefinedProperties()
		propvalues = userprops.getPropertyValues()
		for userprop in propvalues:
			# Имя автора оригинала
			if (self.stringEqualsWithoutCase(userprop.Name, 'X-Original-Author')):
				origauthor = str(userprop.Value)
			# Название оригинала
			elif (self.stringEqualsWithoutCase(userprop.Name, 'X-Original-Name')):
				origname = str(userprop.Value)
			# Строка URL для доступа к оригиналу
			elif (self.stringEqualsWithoutCase(userprop.Name, 'X-Original-URL')):
				origurl = str(userprop.Value)
			# Дата создания оригинала
			elif (self.stringEqualsWithoutCase(userprop.Name, 'X-Original-Date')):
				origdate = self.convertDate(userprop.Value)
		# Вывод заголовка
		htmlfile.write('<html>\n')
		htmlfile.write('<head>\n')
		htmlfile.write('	<meta content="text/html; charset=koi8-r" http-equiv="Content-Type">\n')
		htmlfile.write('	<title>' + document.DocumentProperties.Title + '</title>\n')
		htmlfile.write('	<meta name="description" content="' + document.DocumentProperties.Subject + '">\n')
		htmlfile.write('	<meta name="Keywords" content="' + keywords + '">\n')
		htmlfile.write('	<base href="http://rus-linux.net">\n')
		htmlfile.write('<style>\n')
		htmlfile.write('tt {color: brown; font-size: 12; font-family: Monospace}\n')
		htmlfile.write('pre {color: brown; font-size: 12; font-family: Monospace}\n')
		htmlfile.write('textarea {color: brown; font-size: 12; font-family: Monospace}\n')
		htmlfile.write('p {text-align: justify; FONT-SIZE: 12px; COLOR: #000000; FONT-FAMILY: Verdana; font-weight: normal}\n')
		htmlfile.write('a {color: blue; text-decoration: none }\n')
		htmlfile.write('red {color: red }\n')
		htmlfile.write('</style>\n')
		htmlfile.write('</head>\n\n')
		
		htmlfile.write('<body><h1>' + document.DocumentProperties.Title + '</h1>\n\n')
		
		htmlfile.write('Оригинал: <a href="' + origurl + '">' + origname + '</a>\n')
		htmlfile.write('<br>Автор(ы): ' + origauthor + '\n')
		htmlfile.write('<br>Дата публикации: ' + origdate + '\n')
		htmlfile.write('<br>Перевод: ' + document.DocumentProperties.Author + '\n')
		htmlfile.write('<br>Дата перевода: ' + self.convertDate(document.DocumentProperties.CreationDate) + '\n\n')
		# Данные для специального форматирования
		# Многострочные параграфы
		lastparstyle = 'Standard'
		lastparmultiline = False
		# Списки
		listlastlevel = -1
		# Обход параграфов документа
		parenum = document.Text.createEnumeration()
		while parenum.hasMoreElements():
			# Новый параграф
			par = parenum.nextElement()
			if par.supportsService("com.sun.star.text.Paragraph"):
				if (par.NumberingStyleName != ''):
					# Элемент списка
					if (listlastlevel == -1):
						# Первый элемент списка
						i = 0
						while i <= par.NumberingLevel:
							# При нумерации используется начальное значение
							if (par.NumberingStartValue == -1):
								htmlfile.write('<ol style="font-size:12px;">\n')
							else:
								htmlfile.write('<ol start="' + str(par.NumberingStartValue) + '" style="font-size:12px;">\n')
							i += 1
						htmlfile.write('<li>\n')
					elif (listlastlevel == par.NumberingLevel):
						# Следующий элемент списка
						htmlfile.write('<li>\n')
					elif (listlastlevel < par.NumberingLevel):
						# Элемент списка следующего уровня
						i = 0
						while i < (par.NumberingLevel - listlastlevel):
							# При нумерации используется начальное значение
							if (par.NumberingStartValue == -1):
								htmlfile.write('<ol style="font-size:12px;">\n')
							else:
								htmlfile.write('<ol start="' + str(par.NumberingStartValue) + '" style="font-size:12px;">\n')
							i += 1
						htmlfile.write('<li>\n')
					elif (listlastlevel > par.NumberingLevel):
						# Элемент списка предыдущего уровня
						i = 0
						while i < (listlastlevel - par.NumberingLevel):
							htmlfile.write('</ol>\n')
							i += 1
						htmlfile.write('<li>\n')
					# Сохранение значения уровня
					listlastlevel = par.NumberingLevel	
				elif (listlastlevel != -1):
					# Закрытие списка		
					i = listlastlevel
					while i >= 0:
						htmlfile.write('</ol>\n')
						i -= 1
					listlastlevel = -1
				
				if (par.ParaStyleName == 'Preformatted Text'):
					# Специальным образом отформатированный текст
					if (lastparmultiline):
						if (lastparstyle != 'Preformatted Text'):
							# Закрытие предыдущего многострочного параграфа
							if (lastparstyle == 'Frame contents'):
								htmlfile.write('\n</textarea>\n\n')
							# Добавление тэга нового многострочного параграфа
							htmlfile.write('<p>\n<pre>\n')
							lastparstyle = 'Preformatted Text'
							lastparmultiline = True
						else:
							# Добавление символа перехода на новую строку многострочного параграфа
							htmlfile.write('\n')
					else:
						# Добавление тэга нового многострочного параграфа
						htmlfile.write('<p>\n<pre>\n')
						lastparstyle = 'Preformatted Text'
						lastparmultiline = True
				elif (par.ParaStyleName == 'Frame contents'):
					# Содержимое врезки
					if (lastparmultiline):
						if (lastparstyle != 'Frame contents'):
							# Закрытие предыдущего многострочного параграфа
							if (lastparstyle == 'Preformatted Text'):
								htmlfile.write('\n</pre>\n</p>\n\n')
							# Добавление тэга нового многострочного параграфа
							htmlfile.write('<textarea cols="50" wrap="OFF" style="width:100%" rows="10">\n')
							lastparstyle = 'Frame contents'
							lastparmultiline = True
						else:
							# Добавление символа перехода на новую строку многострочного параграфа
							htmlfile.write('\n')
					else:
						# Добавление тэга нового многострочного параграфа
						htmlfile.write('<textarea cols="50" wrap="OFF" style="width:100%" rows="10">\n')
						lastparstyle = 'Frame contents'
						lastparmultiline = True
				elif (par.ParaStyleName == 'Heading'):
					# Заголовок
					if (lastparmultiline):
						# Закрытие предыдущего многострочного параграфа
						if (lastparstyle == 'Preformatted Text'):
							htmlfile.write('\n</pre>\n</p>\n\n')
						elif (lastparstyle == 'Frame contents'):
							htmlfile.write('\n</textarea>\n\n')
					# Добавление тэга
					htmlfile.write('<h2>')
					lastparstyle = 'Heading'
					lastparmultiline = False
				else:
					# Простой параграф
					if (lastparmultiline):
						# Закрытие предыдущего многострочного параграфа
						if (lastparstyle == 'Preformatted Text'):
							htmlfile.write('\n</pre>\n</p>\n\n')
						elif (lastparstyle == 'Frame contents'):
							htmlfile.write('\n</textarea>\n\n')
					# Добавление тэга
					htmlfile.write('<p>\n')
					lastparstyle = 'Standard'
					lastparmultiline = False
				# Обход фрагментов параграфа
				textenum = par.createEnumeration()
				while textenum.hasMoreElements():
					text = textenum.nextElement()
					# Специальное форматирование
					# Перечеркнутый текст
					if hasattr(text, "CharStrikeout"):
						if (text.CharStrikeout):
							htmlfile.write('<strike>')
					# Подчеркнутый текст
					if (text.CharUnderline == SINGLE):
						htmlfile.write('<u>')
					# Жирный текст
					if (text.CharWeight == BOLD):
						htmlfile.write('<b>')
					# Наклонный текст
					if (text.CharPosture == ITALIC):
						htmlfile.write('<i>')
					# Гиперссылка
					if hasattr(text, "HyperLinkURL"):
						if (text.HyperLinkURL != ''):
							htmlfile.write('<a href="' + text.HyperLinkURL + '" rel="nofollow">')
					
					# Запись фрагмента в результирующий файл
					if hasattr(text, "CharShadowed"):
						if text.CharShadowed:
							# Специальное выделение текста, отмеченного как текст с тенью
							htmlfile.write('<tt>' + html.escape(text.String) + '</tt>')
						else:
							htmlfile.write(html.escape(text.String))
					else:
						htmlfile.write(html.escape(text.String))
					
					# Закрытие тэгов форматирования
					# Гиперссылка
					if hasattr(text, "HyperLinkURL"):
						if (text.HyperLinkURL != ''):
							htmlfile.write('</a>')
					# Наклонный текст
					if (text.CharPosture == ITALIC):
						htmlfile.write('</i>')
					# Жирный текст
					if (text.CharWeight == BOLD):
						htmlfile.write('</b>')
					# Подчеркнутый текст
					if (text.CharUnderline == SINGLE):
						htmlfile.write('</u>')
					# Перечеркнутый текст
					if hasattr(text, "CharStrikeout"):
						if (text.CharStrikeout):
							htmlfile.write('/<strike>')
			
				# Закрытие тэга параграфа
				if (par.ParaStyleName == 'Heading'):
					htmlfile.write('</h2>\n')
				elif (par.ParaStyleName == 'Standard'):
					htmlfile.write('\n</p>\n')
					
				# Закрытие тэга элемента списка
				if (listlastlevel != -1):
					# Список может состоять исключительно из отдельных параграфов при любом форматировании
					# Закрытие предыдущего многострочного параграфа
					if (lastparmultiline):
						if (lastparstyle == 'Preformatted Text'):
							htmlfile.write('\n</pre>\n</p>\n\n')
						elif (lastparstyle == 'Frame contents'):
							htmlfile.write('\n</textarea>\n\n')
						lastparstyle = 'Standard'
						lastparmultiline = False
					htmlfile.write('</li>\n\n')
				else:
					htmlfile.write('\n')
			elif par.supportsService("com.sun.star.text.TextTable"):
				# Закрытие предыдущего многострочного параграфа
				if (lastparmultiline):
					if (lastparstyle == 'Preformatted Text'):
						htmlfile.write('\n</pre>\n</p>\n\n')
					elif (lastparstyle == 'Frame contents'):
						htmlfile.write('\n</textarea>\n\n')
					lastparstyle = 'Standard'
					lastparmultiline = False
				# Закрытие списка
				if (listlastlevel != -1):
					i = listlastlevel
					while i >= 0:
						htmlfile.write('</ol>\n')
						i -= 1
					listlastlevel = -1
				# Таблица
				# Объекты столбцов и строк
				tablerows = par.getRows()
				tablecols = par.getColumns()
				# Количество столбцов и строк
				rowcount = tablerows.getCount()
				colcount = tablecols.getCount()
				
				htmlfile.write('<p>\n<table border="1" style="font-size:12px;">\n')
				# Обход строк
				for rowid in range(0, rowcount):
					htmlfile.write('<tr>\n')
					# Обход столбцов
					for colid in range(0, colcount):
						# Преобразование номеров столбца и строки в номер ячейки
						colnum = colid
						colname = ''
						letterrange = ord('Z') - ord('A') + 1
						# Формирование многобуквенных имен ячеек
						while (colnum > 0):
							curcolnum = colnum % letterrange
							colname = chr(curcolnum + ord('A')) + colname
							colnum = (colnum - curcolnum) / letterrange
						cellname = colname + str(rowid + 1)
						# Получение доступа к объекту ячейки по имени
						cell = par.getCellByName(cellname)
						# Корректная обработка пустых ячеек
						if hasattr(cell, "String"):
							if (rowid == 0):
								htmlfile.write('\t<th>' + cell.String + '</th>\n')
							else:
								htmlfile.write('\t<td>' + cell.String + '</td>\n')
						else:
							htmlfile.write('\t<td>&nbsp;</td>\n')
					htmlfile.write('</tr>\n')
				htmlfile.write('</table>\n</p>\n\n')
		
		# Закрытие предыдущего многострочного параграфа
		if (lastparmultiline):
			if (lastparstyle == 'Preformatted Text'):
				htmlfile.write('\n</pre>\n</p>\n\n')
			elif (lastparstyle == 'Frame contents'):
				htmlfile.write('\n</textarea>\n\n')
			lastparstyle = 'Standard'
			lastparmultiline = False
		# Закрытие списка
		if (listlastlevel != -1):
			i = listlastlevel
			while i >= 0:
				htmlfile.write('</ol>\n')
				i -= 1
			listlastlevel = -1
		# Закрытие документа HTML
		htmlfile.write('</body>\n</html>') 
		# Закрытие результирующего файла
		htmlfile.flush()
		htmlfile.close()
		
# Регистрация реализации службы
g_ImplementationHelper = unohelper.ImplementationHelper()

g_ImplementationHelper.addImplementation( \
		RusLinuxJob,                                # Имя класса UNO
		"org.openoffice.comp.pyuno.exp.RusLinux",   # Имя реализации
		("com.sun.star.task.Job",),)                # Список реализованных служб
